Package: cellkeyperturbation
Title: Cell Key Perturbation
Version: 3.0.0
Authors@R: c(
    person("Iain", "Dove", email = "SMLHelp@ons.gov.uk", 
    role = c("aut", "cph")),
    person("Ahmet", "Aydin", email = "SDC.Queries@ons.gov.uk", 
    role = c("aut", "cre")))
Description: Provides functions to generate frequency tables and apply cell key
    perturbation to protect against statistical disclosure in tabular outputs.
    The implemented methods are described in "Cell Key Perturbation User Guide"
    <https://github.com/ONSdigital/cell-key-perturbation-R/blob/main/documentation/SML_UserDoc_CKP_R.md>.
    Developed at the UK Office for National Statistics.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: bigrquery, DBI, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: data.table
Depends: R (>= 3.5.0)
LazyData: true
VignetteBuilder: knitr
URL: https://github.com/ONSdigital/cell-key-perturbation-R
BugReports: https://github.com/ONSdigital/cell-key-perturbation-R/issues
NeedsCompilation: no
Packaged: 2026-05-21 10:57:32 UTC; aydina
Author: Iain Dove [aut, cph],
  Ahmet Aydin [aut, cre]
Maintainer: Ahmet Aydin <SDC.Queries@ons.gov.uk>
Repository: CRAN
Date/Publication: 2026-05-28 11:40:06 UTC
Built: R 4.5.2; ; 2026-05-28 16:13:52 UTC; unix
